import { Link } from "react-router-dom";
import { Message, User, Channel, Users, Emojis } from "../types";
import { useState, createRef, useEffect } from "react";
import axios from "axios";

import "../styles/MessageComponent.css";

export default function MessageComponent({ message, user, channel }: {
        message: Message;
        user: User | undefined;
        channel: Channel | undefined;
    }) {
    const [reply, setReply] = useState(false);
    const [replyContent, setReplyContent] = useState("");
    const [searchedUsers, setSearchedUsers] = useState<Users>([]);
    const [searchedEmojis, setSearchedEmojis] = useState<Emojis>([]);
    const ref = createRef<HTMLInputElement>();
    const token = localStorage.getItem("token");

    function getMessageArray(message: Message) {
        const array = []
        var string = ''
        message.content.split(" ").forEach((word) => {
            if (word.startsWith("@") && message.mentions.find((mention) => `@${mention.username}` === word)) {
                array.push(string)
                string = ''
                array.push(<Link to={`/u/${word.substring(1)}`}>{word}</Link>)
            } else if (word.startsWith("https://") || word.startsWith("http://")) {
                array.push(string)
                string = ''
                array.push(<Link to={word}>{word}</Link>)
            } else if (word.startsWith(":") && word.endsWith(":") && word.length > 2) {
                array.push(string)
                string = ''
                const emojiName = word.substring(1, word.length - 1);
                array.push(<img src={`/api/emojis/${emojiName}`} alt={emojiName} className="message-emoji" />)
            } else {
                string += word + " "
            }
        });
        array.push(string)
        return array.filter((word) => word !== '');
    }

    function handleReply(e: React.FormEvent<HTMLFormElement>, messageId: number) {
        e.preventDefault();
        axios
            .post(`/api/messages/${messageId}/reply`, {
                token: token,
                message: replyContent,
            })
            .then(() => {
                setReplyContent("");
                setReply(false);
                setSearchedUsers([]);
                setSearchedEmojis([]);
            })
            .catch((err) => {
                console.error(err.response.data);
            });
    }

    function handleDelete() {
        if (!window.confirm("Are you sure you want to delete this message?")) {
            return;
        }
        axios
            .post(`/api/messages/${message.id}/delete`, {
                token: token,
            })
            .catch((err) => {
                console.error(err.response.data);
            });
    }

    useEffect(() => {
        const words = replyContent.toString().split(" ");
        const lastWord = words[words.length - 1];
    
        if (lastWord?.startsWith("@")) {
            const username = lastWord.slice(1);
            axios
                .get(`/api/users?search=${username}`)
                .then((res) => {
                    setSearchedUsers(res.data);
                })
                .catch((err) => {
                    console.error(err.response.data);
                });
        } else {
            setSearchedUsers([]);
        }

        if (lastWord?.startsWith(":")) {
            const emojiName = lastWord.slice(1);
            axios
                .get(`/api/emojis?search=${emojiName}`)
                .then((res) => {
                    setSearchedEmojis(res.data);
                })
                .catch((err) => {
                    console.error(err.response.data);
                });
        } else {
            setSearchedEmojis([]);
        }
    }, [replyContent]);
    
    return (
        <div key={message.id} className="message">
            <div className="message-content">
                <img src={`/api/users/${message.username}/pfp`} alt="" className="message-user-pfp" />
                <div className="message-content-right">
                    <span><Link to={`/u/${message.username}`}>{message.username}</Link> {new Date(message.date * 1000).toLocaleString()}</span>
                    <div className="message-text">
                        {getMessageArray(message).map((word, index) => {
                            return <span key={index}>{word} </span>;
                        })}
                    </div>
                </div>
            </div>
            <div className="message-attachments">
                {message.attachments?.map((attachment) => (
                    <img src={`/api/attachments/${attachment.file_name}`} alt="attachment" className="message-attachment" />
                ))}
            </div>
            {((channel && channel?.owner_id == user?.id) || user?.admin == 1 || user?.id == message.user_id) && (
                <button onClick={handleDelete} className="forum-button">Delete</button>
            )}
            {channel && user && !reply && (
                <button onClick={() => setReply(true)} className="forum-button">Reply</button>
            )}
            {reply && (
                <form onSubmit={(e) => handleReply(e, message.id)} className="form-horizontal">
                    <input 
                        className="forum-input"
                        type="text"
                        placeholder="Message"
                        value={replyContent}
                        onChange={(e) => setReplyContent(e.target.value)}
                        ref={ref}
                    />
                    <button type="submit" className="forum-button">Send</button>
                    {searchedUsers.length > 0 && (
                        <div className="input-menu">
                            {searchedUsers.map((user) => (
                                <div key={user.id} className="input-menu-item">
                                    <Link to={`/u/${user.username}`}>{user.username}</Link>
                                    <button
                                        className="forum-button"
                                        type="button"
                                        onClick={() => {
                                            setReplyContent(
                                                replyContent.split(" ").slice(0, -1).join(" ") + " @" + user.username + " "
                                            );
                                            setSearchedUsers([]);
                                            ref.current?.focus();
                                        }}
                                    >
                                        Mention
                                    </button>
                                </div>
                            ))}
                        </div>
                    )}
                    {searchedEmojis.length > 0 && (
                        <div className="input-menu">
                            {searchedEmojis.map((emoji) => (
                                <div key={emoji.id} className="input-menu-item">
                                    <img src={`/api/emojis/${emoji.name}`} alt={emoji.name} className="emoji" />
                                    <span>:{emoji.name}:</span>
                                    <button
                                        className="forum-button"
                                        type="button"
                                        onClick={() => {
                                            setReplyContent(
                                                replyContent.split(" ").slice(0, -1).join(" ") + " :" + emoji.name + ": "
                                            );
                                            setSearchedEmojis([]);
                                            ref.current?.focus();
                                        }}
                                    >
                                        Add Emoji
                                    </button>
                                </div>
                            ))}
                        </div>
                    )}
                    <button onClick={() => setReply(false)} className="forum-button">Cancel</button>
                </form>
            )}
            {message.has_replies == true && (
                <div className="message-replies">
                    {message.replies.map((reply) => (
                        <MessageComponent message={reply} user={user} channel={channel} key={reply.id} />
                    ))}
                </div>
            )}
            {!channel && (
                <p>in: <Link to={`/c/${message.channel_name}`}>{message.channel_name}</Link></p>
            )}
        </div>
    );
}