import { useState, useEffect } from "react"
import { Link } from "react-router-dom"
import { User, Users } from "../types"
import axios from "axios"
import TopBar from "../components/TopBar"

import "../styles/UsersPage.css"

export default function UsersPage({socket}: {socket: WebSocket}) {
    const [users, setUsers] = useState<Users>();
    const [search, setSearch] = useState<string>("");
    const [token, setToken] = useState<string>("");
    const [thisUser, setThisUser] = useState<User>();

    function deleteUser(username: string) {
        if (!window.confirm(`Are you sure you want to delete ${username}?`)) {
            return;
        }
        axios
            .post(`/api/users/${username}/delete`, { token })
            .catch((err) => {
                console.error(err.response);
            });
    }

    useEffect(() => {
        const token = localStorage.getItem("token")

        if (token) {
            setToken(token)
            axios
                .post("/api/auth/me", {
                    token: token
                })
                .then((res) => {
                    setThisUser(res.data)
                })
        }
        axios
            .get("/api/users")
            .then((res) => {
                setUsers(res.data)
            })
            .catch((err) => {
                console.error(err.response)
            })
    }, [])

    useEffect(() => {
        socket.addEventListener('message', function (event) {
            const data = JSON.parse(event.data);
            if (data.type === "new_user" || data.type === "delete_user") {
                axios
                    .get("/api/users")
                    .then((res) => {
                        setUsers(res.data)
                    })
                    .catch((err) => {
                        console.error(err.response)
                    })
            }
        });
    }, [])

    if (!users) {
        return (
            <div className="forum-page">
                <TopBar user={thisUser} />
                <div className="forum-section">
                    <h2>Users</h2>
                    <p>Loading...</p>
                </div>
            </div>
        )
    }

    return (
        <div className="forum-page">
            <TopBar user={thisUser} />
            <div className="forum-section">
                <h2>Users</h2>
                <input
                    className="forum-input"
                    type="text"
                    placeholder="Search users"
                    value={search}
                    onChange={(e) => setSearch(e.target.value)}
                />
                <div className="forum-users">
                    {users?.sort().filter((channel) => channel.username.toLowerCase().includes(search.toLowerCase())).map((user) => (
                        <div key={user.id} className="forum-user">
                            <div className="forum-user-left">
                                <img src={`/api/users/${user.username}/pfp`} alt="pfp" className="forum-user-pfp" />
                                <div className="forum-user-info">
                                    <div className="forum-user-title">
                                        <h3><Link to={`/u/${user.username}`}>{user.username}</Link></h3>
                                        {user.admin == 1 && <span className="forum-user-admin">Admin</span>}
                                    </div>
                                    {user.description ? (
                                        <p className="forum-user-description">{user.description}</p>
                                    ) : (
                                        <p className="forum-user-description">No description</p>
                                    )}
                                </div>
                            </div>
                            {thisUser?.admin == 1 && (
                                <button onClick={() => deleteUser(user.username)} className="forum-button"> 
                                    Delete
                                </button>
                            )}
                        </div>
                    ))}
                </div>
            </div>
        </div>
    )
}